"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.analizeSubcommandGroupParsed = exports.analizeSubCommandParsed = exports.parseSlashSubcommand = exports.isCommandOptionsUpdated = exports.subCommandsGroupRegistry = exports.subCommandsRegistry = void 0;
const framework_1 = require("@sapphire/framework");
const discord_js_1 = require("discord.js");
const builders_1 = require("@discordjs/builders");
/**
 * **Registry**
 *
 * Subcommands registry.
 *
 * @since 1.0.0
 */
exports.subCommandsRegistry = new discord_js_1.Collection();
/**
 * **Registry**
 *
 * Subcommands group registry.
 *
 * @since 1.0.0
 */
exports.subCommandsGroupRegistry = new discord_js_1.Collection();
/**
 * **Subcommands compare**
 *
 * Compare two subcommand builders.
 *
 * @param c1 SubcommandBuilder 1
 * @param c2 SubcommandBuilder 2
 * @returns boolean
 * @since 1.0.0
 */
const isCommandOptionsUpdated = (c1, c2) => JSON.stringify(c1.toJSON()) !== JSON.stringify(c2.toJSON());
exports.isCommandOptionsUpdated = isCommandOptionsUpdated;
/**
 * Parse slash subcommand option
 * @param subcommand subcommand json or subcommand builder class.
 * @returns subcommand builder
 *
 * @since 2.0.0
 */
const parseSlashSubcommand = (subcommand) => (typeof subcommand === 'function' ? subcommand(new builders_1.SlashCommandSubcommandBuilder(), framework_1.container) : subcommand);
exports.parseSlashSubcommand = parseSlashSubcommand;
/**
 * **Register subcommands in subcommands registry**
 *
 * Method that parses and registers subcommands in parent commands.
 *
 * @param piece Class of the command to be used as a subcommand of the parent command.
 * @param parentCommandName Name of the parent command.
 * @param subcommand Builder of the subcommand to be analyzed in the subcommand register.
 * @returns	Command class
 * @since 1.0.0
 */
const analizeSubCommandParsed = (piece, parentCommandName, subcommand) => {
    const subcommandParsed = (0, exports.parseSlashSubcommand)(subcommand);
    const subcommandsRegistry = exports.subCommandsRegistry.get(parentCommandName);
    if (!subcommandsRegistry) {
        exports.subCommandsRegistry.set(parentCommandName, new discord_js_1.Collection().set(subcommandParsed.name, {
            slashCommand: subcommandParsed,
            commandPiece: piece
        }));
        framework_1.container.logger.debug(`[Subcommands-Plugin]: The parent command "${parentCommandName}" has been registered and the subcommand "${subcommandParsed.name}" has been registered.`);
        return piece;
    }
    const command = subcommandsRegistry.get(subcommandParsed.name);
    if (!command) {
        subcommandsRegistry.set(subcommandParsed.name, { slashCommand: subcommandParsed, commandPiece: piece });
        framework_1.container.logger.debug(`[Subcommands-Plugin]: The subcommand "${subcommandParsed.name}" has been registered in the parent command "${parentCommandName}".`);
        return piece;
    }
    const commandsCompare = (0, exports.isCommandOptionsUpdated)(command.slashCommand, subcommandParsed);
    subcommandsRegistry.delete(subcommandParsed.name);
    subcommandsRegistry.set(subcommandParsed.name, { slashCommand: subcommandParsed, commandPiece: piece });
    framework_1.container.logger.debug(`[Subcommands-Plugin]: The subcommand "${subcommandParsed.name}" has been updated in the parent command ${parentCommandName} ${commandsCompare ? 'with new options' : 'without new options'}.`);
    const parentCommand = framework_1.container.stores.get('commands').get(parentCommandName);
    if (parentCommand) {
        if (!parentCommand?.parsedSubcommandMappings) {
            framework_1.container.logger.error(`[Subcommands-Plugin]: The parent command ${parentCommandName} possibly does not extend the plugin Subcommand class because the "parsedSubcommandMappings" property was not found in the parent command required for the subcommand ${subcommandParsed.name}.`);
            return piece;
        }
        if (commandsCompare)
            void parentCommand.reload();
        const subcommand = parentCommand.parsedSubcommandMappings.find((s) => s.name === subcommandParsed.name && s.type === 'method');
        if (subcommand) {
            if (piece.chatInputRun)
                subcommand.chatInputRun = (i, c) => piece.chatInputRun(i, c);
            // TODO: Support for message commands coming soon
            // if (piece.messageRun) subcommand.messageRun = (m, a, c) => piece.messageRun!(m, a, c);
        }
    }
    else {
        framework_1.container.logger.warn(`[Subcommands-Plugin]: Could not get information from the parent command ${parentCommandName} for the subcommand ${subcommandParsed.name}.`);
    }
    return piece;
};
exports.analizeSubCommandParsed = analizeSubCommandParsed;
/**
 * **Register subcommands in subcommands group registry**
 *
 * Method that parses and registers subcommands in parent commands.
 *
 * @param piece Class of the command to be used as a subcommand of the parent command.
 * @param parentCommandName Name of the parent command.
 * @param groupName Name of the subcommand group.
 * @param subcommand Builder of the subcommand to be analyzed in the subcommand register.
 * @returns	Command class
 * @since 1.0.0
 */
const analizeSubcommandGroupParsed = (piece, parentCommandName, groupName, subcommand) => {
    const subcommandParsed = (0, exports.parseSlashSubcommand)(subcommand);
    const subcommandsGroup = exports.subCommandsGroupRegistry.get(parentCommandName);
    if (!subcommandsGroup) {
        exports.subCommandsGroupRegistry.set(parentCommandName, new discord_js_1.Collection().set(groupName, new discord_js_1.Collection().set(subcommandParsed.name, {
            slashCommand: subcommandParsed,
            commandPiece: piece
        })));
        framework_1.container.logger.debug(`[Subcommands-Group-Plugin]: The parent command "${parentCommandName}" has been registered and the group "${groupName}" has been created with the registered "${subcommandParsed.name}" command.`);
        return piece;
    }
    const group = subcommandsGroup.get(groupName);
    if (!group) {
        subcommandsGroup.set(groupName, new discord_js_1.Collection().set(subcommandParsed.name, {
            slashCommand: subcommandParsed,
            commandPiece: piece
        }));
        framework_1.container.logger.debug(`[Subcommands-Group-Plugin]: The group "${groupName}" has been registered with the command ${subcommandParsed.name} registered in the parent command "${parentCommandName}".`);
        return piece;
    }
    const commandGroup = group.get(subcommandParsed.name);
    if (!commandGroup) {
        group.set(subcommandParsed.name, { slashCommand: subcommandParsed, commandPiece: piece });
        framework_1.container.logger.debug(`[Subcommands-Group-Plugin]: The command "${subcommandParsed.name}" has been registered in the group "${groupName}" of the parent command "${parentCommandName}".`);
        return piece;
    }
    const commandsGroupCompare = (0, exports.isCommandOptionsUpdated)(commandGroup.slashCommand, subcommandParsed);
    group.delete(commandGroup.slashCommand.name);
    group.set(subcommandParsed.name, { slashCommand: subcommandParsed, commandPiece: piece });
    framework_1.container.logger.debug(`[Subcommands-Group-Plugin]: The command ${subcommandParsed.name} has been updated in the group "${groupName}" of the parent command "${parentCommandName}" ${commandsGroupCompare ? 'with new options' : 'without new options'}.`);
    const parentCommand = framework_1.container.stores.get('commands').get(parentCommandName);
    if (parentCommand) {
        if (!parentCommand?.parsedSubcommandMappings) {
            framework_1.container.logger.error(`[Subcommands-Group-Plugin]: The parent command ${parentCommandName} possibly does not extend the plugin Subcommand class because the "parsedSubcommandMappings" property was not found in the parent command required for the subcommand ${subcommandParsed.name} of the subcommand group ${groupName}.`);
            return piece;
        }
        if (commandsGroupCompare)
            void parentCommand.reload();
        const subcommandGroup = parentCommand.parsedSubcommandMappings.find((s) => s.name === groupName && s.type === 'group');
        if (subcommandGroup) {
            const subcommand = subcommandGroup.entries.find((s) => s.name === subcommandParsed.name && s.type === 'method');
            if (subcommand) {
                if (piece.chatInputRun)
                    subcommand.chatInputRun = (i, c) => piece.chatInputRun(i, c);
                // TODO: Support for message commands coming soon
                // if (piece.messageRun) subcommand.messageRun = (m, a, c) => piece.messageRun!(m, a, c);
            }
        }
    }
    else {
        framework_1.container.logger.warn(`[Subcommands-Group-Plugin]: Could not get information from the parent command ${parentCommandName} for the subcommand ${subcommandParsed.name} of the subcommand group ${groupName}.`);
    }
    return piece;
};
exports.analizeSubcommandGroupParsed = analizeSubcommandGroupParsed;
//# sourceMappingURL=functions.js.map